/*
    SageTV backup tool
    
    Last Modification: 12 Mar 2011
               Author: Derek Battams <derek AT battams DOT ca>
    
    YOU CANNOT RUN THIS TASK VIA THE SAGETV PLUGIN TASK CLIENT! YOU MUST RUN IT
    OUTSIDE OF SAGETV! YOU'VE BEEN WARNED!!
    
    You cannot run this within the plugin version of the task client because this
    script stops SageTV, which means that it also stops the plugin version of the
    task client.  Therefore, if you're running this task as part of SJQv4 then you
    must run this task on a standalone client.  The other alternative is to disable
    the stopping of SageTV before backing up, but this is HIGHLY discouraged (see
    below).  Basically, setup a second task client on another port on the same host
    as the SageTV server to run this task.
    
    Exception to the above rule: If you're running on a SageClient and you plan to
    restart the server remotely then you can run via a SageClient plugin task client
    ONLY.  See below for remote SageTV restarts (advanced feature).
    
    THIS SCRIPT ASSUMES IT IS RUNNING ON THE SAME HOST AS SAGETV SERVER
    
    If you can remotely restart SageTV then you could run this on a remote host,
    just update the command lines in the config section.  SageTV on Linux via ssh
    command invocation would be a good candidate for remote execution.  There are
    ways to do the same on Windows (Google 'psexec'), but that's all I'll say 
    about remote restarting of SageTV.  I run it locally on Windows by using a
    second task client (running as Windows service).
        
    Use this Groovy script to stop SageTV, backup the entire application directory
    then restart SageTV.  This script should be called only after a test script
    for the task has determined that it is "safe" to stop SageTV and proceed with
    the backup (or you could add those "safety" checks to this script if running it
    outside of SJQv4).
    
    You must configure this script for your environment!
    
    Point the src variable to the base installation directory of SageTV.
    Everything under this directory will be backed up.
    
    Point the target variable to the base directory where backups should be
    stored.  Backups are placed in a subdir named yyyymmdd.
    
    Ensure the stopCmd and startCmd variables are valid for your system.  They
    are preconfigured to start/stop Sage via the Windows service mechanism.  If
    you don't run SageTV on Windows or you don't run it as a service then  you'll
    need to modify these variables.
    
    You can optionally run a backup without first stopping SageTV by setting the
    restartSage variable to false, however, I don't recommend this as copying files
    that are actively being used by SageTV is probably not a good idea.  I haven't
    ever tested this with restartSage=false and I don't ever intend to. :)
    
    TO DO/WISH LIST
    
    * Allow for optional zipping of backup directory
*/

/****** CONFIG BELOW ******/
def testMode         = false                                // Should always run this in test mode before putting it in production!
def src              = new File("C:/Program Files/SageTV/") // Source directory to be backed up
def target           = new File("D:/fullbackups/")          // Target base directory; backup will go into a subdir named after today's date
def numBackupsToKeep = 2                                    // Number of old backups to keep
def restartSage      = true                                 // If true, stop Sage before backing up and start it after backing up
def stopCmd          = ["net.exe", "stop", "sagetv"]        // Command to stop SageTV
def startCmd         = ["net.exe", "start", "sagetv"]       // Command to start SageTV
def sleepTime        = 300                                  // How many seconds does SageTV need to restart? BE CONSERVATIVE HERE!!
/****** END CONFIG ******/

// DO NOT EDIT BELOW THIS LINE
// But if you do, send me your patches! ;)

def todayDir = new File(target, new Date().format("yyyyMMdd"))

import org.apache.commons.io.FileUtils
import org.apache.commons.io.FilenameUtils

if(!testMode && !target.isDirectory()) {
   println "Backup directory does not exist! [${target}]"
   return 2
}

if(!testMode && todayDir.isDirectory()) {
   println "Looks like a backup has already been done for today!"
   return 2
}

def stopped = true
if(restartSage) {
   print "Stopping SageTV... "
   if(!testMode) {
       def stdout = new StringWriter()
       def stderr = new StringWriter()
       if(!runCmdLine(stopCmd, stdout, stderr)) {
           println "FAILED"
           System.err.println("CMD: $stopCmd")
           if(stdout.toString().length() > 0)
               System.err.println(stdout.toString())
           if(stderr.toString().length() > 0)
               System.err.println(stderr.toString())
           stopped = false
       } else
           println "DONE"
   } else
       println "SKIPPED (test mode)"
}

print "Backing up '$src' to '$todayDir'... "
def backupSuccess = true
if(!testMode) {
    if(restartSage && !stopped) {
        println "SKIPPED (stop SageTV failed)"
        backupSuccess = false
    } else {
        try {
            FileUtils.copyDirectory(src, todayDir)
            println "DONE"
        } catch(IOException e) {
            println "ERROR: ${e.getMessage()}"
            e.printStackTrace()
            backupSuccess = false
        }
    }
} else
    println "SKIPPED (test mode)"

if(backupSuccess && target.isDirectory() && numBackupsToKeep > 0) {
   def dirs = target.list(new BackupMatcher()).sort().reverse()
   for(def i = numBackupsToKeep; i < dirs.size(); ++i) {
      print "Deleting old backup '${dirs[i]}'... "
      if(!testMode) {
         try {
            FileUtils.deleteDirectory(new File(target, dirs[i]))
            println "DONE"
         } catch(IOException e) {
            println "ERROR: ${e.getMessage()}"
         }
      } else
         println "SKIPPED (test mode)"
   }
} else if(!backupSuccess)
    println "Skipping removal of old backups because the current backup attempt failed!"

if(restartSage) {
   print "Starting SageTV... "
   if(!testMode) {
       if(stopped) {
           def stdout = new StringWriter()
           def stderr = new StringWriter()
           if(!runCmdLine(startCmd, stdout, stderr)) {
               println "FAILED"
               System.err.println("CMD: $stopCmd")
               if(stdout.toString().length() > 0)
                   System.err.println(stdout.toString())
               if(stderr.toString().length() > 0)
                   System.err.println(stderr.toString())
           } else {
               println "DONE"
               print "Waiting for SageTV to start up... "
               sleep 1000L * sleepTime
               println "DONE"
           }
       } else
           println "SKIPPED (SageTV is not stopped)"
   } else
       println "SKIPPED (test mode)"
}

return backupSuccess ? 0 : 2

def runCmdLine(def cmdLine, def stdout, def stderr) {
    def p = cmdLine.execute()
    p.consumeProcessOutput(stdout, stderr)
    p.waitFor()
    return p.exitValue() == 0
}

private class BackupMatcher implements java.io.FilenameFilter {   
   @Override
   boolean accept(File dir, String name) {
      return name ==~ /\d{8}/ && new File(dir, name).isDirectory()
   }
}