/*
    Skip IR tuning for PPV channels
    
    Last Modified: 05 Feb 2011
           Author: Derek Battams <derek AT battams DOT ca>

    Use this script to selectively disable IR tuning for specific (usually PPV) channels.

    When I want to record a PPV movie from my satellite STB I have to manually order the
    movie from the STB, which is fine.  However, when I do this the STB then sets an
    auto tune for the PPV channel.  When Sage tries to tune the channel it seems to always
    interfere with the STB's auto tune process and most of the time the STB does not tune
    to the proper channel and I end up wasting $5.99 for the PPV movie I tried to record.

    This script will disable Sage from sending IR signals to the specified channels.  You
    may specify channels explicitly in the list or by regex.  Channels specified will have
    their physical channel remapped to a meaningless letter, which Sage will ignore resulting
    in Sage sending no IR signals for those channels.
*/

// Do NOT modify these two values!
def KEY_REGEX = "regex"
def KEY_LIST = "list"

/***** CONFIG BELOW *****/

// If specifying a regex, you must prefix it with a '~' i.e. ~/PPV\d+/
// if you don't, the script will fail
// The channel list is a comma separated list of strings i.e. "ABC", "WCBS", "HBO"
def chans = [(KEY_REGEX):~/\d+PPV|PPVHD\d+|SHDPPV\d+|HDPPV\d+/, (KEY_LIST):[]] // Define channels to ignore by regex and/or list
def testMode = true // If true, don't actually remap the channels, just list the channels that would be remapped

/*****  END CONFIG  *****/

/************ DO NOT EDIT BELOW THIS LINE **********/

def nonNumsProp = "mmc/remove_non_numeric_characters_from_channel_change_strings"

def rmNonNums = Configuration.GetServerProperty(nonNumsProp, null)
if(rmNonNums == null || !Boolean.parseBoolean(rmNonNums)) {
   println "You must set the '${nonNumsProp}' to true on your SageTV server in order for this script to work properly!"
   if(!testMode)
       return 1
}

def chansToIgnore = []

def regex = chans.get(KEY_REGEX)
if(regex != null)
    for(Object chan : Database.FilterByMethodRegex(ChannelAPI.GetAllChannels(), "GetChannelName", regex, true, true))
        chansToIgnore << chan

def chanList = chans.get(KEY_LIST)
if(chanList != null)
    for(Object chan : chanList)
        for(Object match : Database.FilterByMethod(ChannelAPI.GetAllChannels(), "GetChannelName", chan.toString(), true))
            chansToIgnore << match

def lineups = Global.GetAllLineups()
for(Object lineup : lineups) {
   for(Object chan : chansToIgnore) {
      if(!ChannelAPI.IsChannelViewableOnLineup(chan, lineup))
         continue
      if(!testMode) {
         ChannelAPI.SetPhysicalChannelMappingForLineup(chan, lineup, "Z_IGNORE")
         println "Remapped '${ChannelAPI.GetChannelName(chan)}' for lineup '$lineup'!"
      } else
         println "Would remap '${ChannelAPI.GetChannelName(chan)}' for lineup '$lineup'!"
   }
}