package com.google.code.sagetvaddons.groovy.api

import sagex.UIContext
import sagex.api.*

/**
 * Various helper/convenience methods related to SageAPI MediaFile objects
 * @author dbattams
 * @version $Id: MediaFileHelpers.groovy 1523 2011-05-16 12:39:54Z derek@battams.ca $
 */
final class MediaFileHelpers {

	/**
	 * Check if a media file is older than the specified age
	 * <p>The age specifier is a special string formatted as such:</p>
	 * <code>nX</code>
	 * <p>Where n is an integer and X is one of the following letters:</p>
	 * <ul>
	 *   <li>S: Seconds</li>
	 *   <li>M: Minutes</li>
	 *   <li>H: Hours</li>
	 *   <li>D: Days</li>
	 *   <li>W: Weeks</li>
	 *   <li>O: mOnths</li>
	 *   <li>Y: Years</li>
	 * </ul>
	 * <p>This method is a convenience method allowing for each checking of the age of media file objects.</p>
	 * <pre>
	 *   // Check is a media file is more than two years old
	 *   def mf // Assigned somewhere above, it is a reference to the media file being examined
	 *   if(MediaFileHelpers.isOlderThan(mf, "2Y")) {
	 *     // It is more than two years old, do whatever
	 *   } else {
	 *     // It is less than two years old, do whatever
	 *   }  
	 * </pre>
	 * @param mf The media file to be checked
	 * @param age The age to check against
	 * @return True if the media file is older than the given age or false otherwise
	 * @throws IllegalArgumentException If the age argument is invalid
	 */
	static boolean isOlderThan(def mf, String age) {
		return MediaFileAPI.GetFileEndTime(mf) < (System.currentTimeMillis() - ageToLong(age))
	}

	/**
	 * Check if a media file is newer than the specified age
	 * <p>See details for <code>MediaFileHelpers.isOlderThan()</code> to learn about the required format of the <code>age</code> string.</p>
	 * @param mf The media file being examined
	 * @param age The age value being compared against
	 * @return True if the media file is newer than the given age or false otherwise
	 * @throws IllegalArgumentException If the age argument is invalid
	 */
	static boolean isNewerThan(def mf, String age) {
		return MediaFileAPI.GetFileEndTime(mf) > (System.currentTimeMillis() - ageToLong(age))
	}

	// Convert an age string into a long suitable for comparisons
	static private long ageToLong(String age) {
		if(!(age ==~ /\d+[SMHDWOY]/))
			throw new IllegalArgumentException("Invalid age! [$age]")
		long units = age.substring(0, age.length() - 1).toLong()
		def type = age.substring(age.length() - 1)
		long multiplier
		switch(type) {
			case "S": multiplier = 1L; break
			case "M": multiplier = 60L; break
			case "H": multiplier = 3600L; break
			case "D": multiplier = 86400L; break
			case "W": multiplier = 7L * 86400L; break
			case "O": multiplier = 30L * 86400L; break // mOnth
			case "Y": multiplier = 365L * 86400L; break
		}
		return 1000L * units * multiplier
	}

	/**
	 * Determine if the given media file is currently being viewed on any client/extender
	 * @param mf The media file being queried
	 * @return True if the media file is currently being viewed somewhere or false otherwise
	 */
	static boolean isBeingViewed(def mf) {
		def connected = []
		connected += Global.GetUIContextNames().toList()
		connected += Global.GetConnectedClients().toList()

		def active = []
		for(def c : connected) {
			def nowPlaying = MediaPlayerAPI.GetCurrentMediaFile(new UIContext(c))
			if(nowPlaying != null)
				active.add(nowPlaying)
		}

		for(def playing : active)
			if(MediaFileAPI.GetMediaFileID(mf) == MediaFileAPI.GetMediaFileID(playing))
				return true
		return false
	}

	/**
	 * Check if the given media file is less than the specified size
	 * <p>The size specifier is a special string formatted as such:</p>
	 * <code>nX</code>
	 * <p>Where n is an integer and X is one of the following letters:</p>
	 * <ul>
	 *   <li>B: Bytes</li>
	 *   <li>K: Kilobytes</li>
	 *   <li>M: Megabytes</li>
	 *   <li>G: Gigabytes</li>
	 *   <li>T: Terabytes</li>
	 * </ul>
	 * <p>This method is a convenience method allowing for easy checking of the size of media file objects.</p>
	 * <pre>
	 *   // Check is a media file is more than 2 GB
	 *   def mf // Assigned somewhere above, it is a reference to the media file being examined
	 *   if(MediaFileHelpers.isSizeGreaterThan(mf, "2G")) {
	 *     // It is more than 2 GB, do whatever
	 *   } else {
	 *     // It is less than 2 GB, do whatever
	 *   }  
	 * </pre>
	 * @param mf The media file object being examined
	 * @param size The size specifier being checked against
	 * @return True if the given media file is larger than the given size specifier or false otherwise
	 * @throws IllegalArgumentException If the size specifier is invalid
	 */
	static boolean isSizeGreaterThan(def mf, String size) {
		return MediaFileAPI.GetSize(mf) > sizeToLong(size)
	}

	/**
	 * Checks if the given media file is smaller than the given size specifier
	 * <p>See the details for {@see MediaFileHelpers#isSizeGreaterThan(Object, String)} to get the required format for the size specifier</p>
	 * @param mf The media file being examined
	 * @param size The size specifier being compared against
	 * @return True if the given media file is smaller than the given size or false otherwise
	 * @throws IllegalArgumentException If the size specifier is invalid
	 */
	static boolean isSizeLessThan(def mf, String size) {
		return MediaFileAPI.GetSize(mf) < sizeToLong(size)
	}

	// Private helper to convert a size specifier to a long suitable for comparisons
	static private long sizeToLong(String size) {
		if(!(size ==~ /\d+[BKMGT]/))
			throw new IllegalArgumentException("Invalid size! [$size]")
		long units = size.substring(0, size.length() - 1).toLong()
		def type = size.substring(size.length() - 1)
		long multiplier
		switch(type) {
			case "B": multiplier = 1L; break
			case "K": multiplier = 1024L; break
			case "M": multiplier = 1024L ** 2; break
			case "G": multiplier = 1024L ** 3; break
			case "T": multiplier = 1024L ** 4; break
		}
		return units * multiplier
	}

	private MediaFileHelpers() {}
}