package com.google.code.sagetvaddons.groovy.bt

import sagex.api.*

/**
 * A BitTorrent favorite; these types of favorites don't record tv from SageTV turners, but instead monitor BT RSS feeds and download new content
 */
abstract public class Favorite {

	/**
	 * The fav prop holding the class name of the favorite being used
	 */
	static public final String PROP_BT_TYPE = "BT_TYPE"

	/**
	 * The fav prop holding the season number of the last episode downloaded
	 */
	static public final String PROP_LAST_SEASON = "BT_LAST_SEASON"

	/**
	 * The fav prop holding the episode number of the last episode downloaded
	 */
	static public final String PROP_LAST_EPISODE = "BT_LAST_EPISODE"

	/**
	 * The fav prop holding the date of the last episode downloaded
	 */
	static public final String PROP_LAST_DATE = "BT_LAST_DATE"

	/**
	 * The fav prop holding the RSS URL to monitor for this favorite
	 */
	static public final String PROP_RSS_URL = "BT_RSS_URL"

	/**
	 * The fav prop holding the URL of a pending torrent to be downloaded
	 */
	static public final String PROP_TORRENT_URL = "BT_TORRENT_URL"

	/**
	 * The fav prop holding the expected file/dir name of the pending torrent being downloaded
	 */
	static public final String PROP_TORRENT_FILE = "TorrentFile"

	/**
	 * The prefix attached to all BT favs stored in SageTV; prevents Sage from actually recording anything for these favs
	 */
	static public final String PROP_PREFIX = "[BT Favorite] "

	def sageFav = null
	def name = null
	def rssFeed = null
	def lastSeason = 0
	def lastEpisode = 0
	def lastDate = 0
	def doneRepack = false

	/**
	 * Constructor
	 * @param favId The SageTV fav id representing this favorite
	 */
	public Favorite(int favId) {
		sageFav = FavoriteAPI.GetFavoriteForID(favId)
		if(sageFav == null)
			throw new RuntimeException("Invalid SageTV favorite ID! [$favId]")
		name = FavoriteAPI.GetFavoriteTitle(sageFav).substring(PROP_PREFIX.length())
		rssFeed = FavoriteAPI.GetFavoriteProperty(sageFav, PROP_RSS_URL)
		if(rssFeed == "")
			throw new RuntimeException("BT favorite has no RSS feed property!")
		try {
			lastSeason = FavoriteAPI.GetFavoriteProperty(sageFav, PROP_LAST_SEASON).toInteger()
			lastEpisode = FavoriteAPI.GetFavoriteProperty(sageFav, PROP_LAST_EPISODE).toInteger()
		} catch(NumberFormatException e) {
			lastSeason = 0
			lastEpisode = 0
		}
		try {
			lastDate = Date.parse("yyyy-MM-dd", FavoriteAPI.GetFavoriteProperty(sageFav, PROP_LAST_DATE))
		} catch(java.text.ParseException e) {
			lastDate = new Date(0)
		}
	}

	/**
	 * Check the RSS feed new content of this favorite
	 * @return True if there's new content, false otherwise; if there is new content then the various internal props of the Sage fav will be updated with the new download info
	 */
	abstract public boolean getNewEpisode()

	/**
	 * Save the state of the object to the Sage favorite object; this is how these objects are persisted
	 */    
	protected void save() {
		if(lastSeason > 0 && lastEpisode > 0) {
			FavoriteAPI.SetFavoriteProperty(sageFav, PROP_LAST_SEASON, lastSeason.toString())
			FavoriteAPI.SetFavoriteProperty(sageFav, PROP_LAST_EPISODE, lastEpisode.toString())
		} else if(lastDate.getTime() > 0)
			FavoriteAPI.SetFavoriteProperty(sageFav, PROP_LAST_DATE, lastDate.format("yyyy-MM-dd"))
	}
}