package com.google.code.sagetvaddons.groovy.net

import java.util.regex.Pattern

import org.apache.commons.net.telnet.TelnetClient

/**
 * <p>Simple class for communicating with SageTV extender devices via their telnet interfaces</p>
 * <p>
 *   Use this class to connect to a SageTV extender and perform commands on the device.  This class
 *   has been tested against HD200 and HD100 devices.  It should work fine on HD300 devices, too.  It
 *   may need some tweaking to work with MediaMVP devices.
 * </p>
 * <p>Use of this class requires that the Apache Commons Net library be on the classpath</p>
 * @author dbattams
 * @version $Id$
 */
class ExtenderClient {
	static private final String RC_WRAPPER = '#@'
	static private final String USER = 'root'
    static private final Pattern RC_REGEX = Pattern.compile("${Pattern.quote(RC_WRAPPER)}\\d+${Pattern.quote(RC_WRAPPER)}")
	
	private TelnetClient clnt
	private InputStream input
	private OutputStream output
	/**
	 * Host/IP of extender device
	 */
	final String host
	
	/**
	 * Constructor
	 * @param host The hostname/IP address of the extender device to connect to
	 * @throws IOException in case of any failure connecting to the device
	 */
	ExtenderClient(String host) throws IOException {
		this.host = host
		
		clnt = new TelnetClient()
		clnt.connect(this.host)
		input = clnt.getInputStream()
		output = clnt.getOutputStream()
		readFromServer()
		writeToServer(USER)
	}
	
	/**
	 * <p>Execute a command on the device</p>
	 * @param cmd The command line to be executed on the device; multiple commands are supported; backgrounding processes is NOT supported
	 * @return The return code of the command line execution
	 * @throws IOException In case of any network error communicating with the device or if the return code cannot be determined from the output
	 */
	int executeCommand(String cmd) throws IOException {
		readFromServer()
		def fullCmd = "$cmd; echo \"${RC_WRAPPER}\$?${RC_WRAPPER}\""
		writeToServer(fullCmd)
		def data = readFromServer()
		for(def line : data.split('\r\n'))
			if(line ==~ RC_REGEX)
				return line.substring(RC_WRAPPER.length(), line.length() - RC_WRAPPER.length()).toInteger()
		throw new IOException('Failed to find RC status in output!')
	}
	
	// Read from the device; don't block if wait = false
	private String readFromServer(boolean wait = false) throws IOException {
		def msg = new StringBuilder()
		def data
		try {
			if(!wait)
				clnt.setSoTimeout(750) // HD100 needs a longer timeout
			else
				clnt.setSoTimeout(0)
			while((data = input.read()) > -1)
				msg.append(data as char)
		} catch(SocketTimeoutException e) {
			if(wait)
				throw e
		}
		return msg.toString()
	}
 
	// Write the to device
	private void writeToServer(String msg) throws IOException {
		output.write("$msg\r\n".getBytes())
		output.flush()
	}
	
	/**
	 * Close the connection to the extender
	 * @throws IOException If there was a network error communicating with the device
	 */
	void disconnect() throws IOException {
		if(clnt) {
			clnt.disconnect()
			clnt = null
		}
	}
	
	@Override
	protected void finalize() {
		try { disconnect() } catch(IOException e) { /* Ignore it */ }
	}
}